Package: MGDrivE2
Type: Package
Title: Mosquito Gene Drive Explorer 2
Version: 2.1.1
Authors@R: c(
    person("Sean L.", "Wu", email = "slwood89@gmail.com", role = c("aut", "cre")),
    person("Jared B.", "Bennett", email = "jared_bennett@berkeley.edu ", role = "aut"),
    person("Héctor Manuel", "Sánchez Castellanos", email = "sanchez.hmsc@berkeley.edu", role = "ctb"),
    person("Tomás M.", "León", email = "tomas.leon@berkeley.edu", role = "ctb"),
    person("Andrew J.", "Dolgert", email = "adolgert@uw.edu", role = "ctb"),
    person("John M.", "Marshall", email = "john.marshall@berkeley.edu", role = "aut"),
    person("Agastya", "Mondal", email= "amondal13@gmail.com", role = "aut")
    )
Maintainer: Sean L. Wu <slwood89@gmail.com>
URL: https://marshalllab.github.io/MGDrivE/docs_v2/index.html,
        https://www.marshalllab.com/
BugReports: https://github.com/MarshallLab/MGDrivE/issues
Description: A simulation modeling framework which significantly extends capabilities from the
    'MGDrivE' simulation package via a new mathematical and computational framework based on stochastic Petri nets.
    For more information about 'MGDrivE', see our publication: Sánchez et al. (2019) <doi:10.1111/2041-210X.13318>
    Some of the notable capabilities of 'MGDrivE2' include: incorporation of human populations,
    epidemiological dynamics, time-varying parameters, and a continuous-time simulation
    framework with various sampling algorithms for both deterministic and stochastic interpretations.
    'MGDrivE2' relies on the genetic inheritance structures provided in package 'MGDrivE', so we
    suggest installing that package initially.
License: GPL-3
Encoding: UTF-8
ByteCompile: true
LazyData: true
Depends: R (>= 3.1.0)
Imports: Matrix, deSolve, statmod
Suggests: MGDrivE, knitr, rmarkdown, markdown, ggplot2
VignetteBuilder: knitr
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2025-09-14 02:17:09 UTC; slwu89
Author: Sean L. Wu [aut, cre],
  Jared B. Bennett [aut],
  Héctor Manuel Sánchez Castellanos [ctb],
  Tomás M. León [ctb],
  Andrew J. Dolgert [ctb],
  John M. Marshall [aut],
  Agastya Mondal [aut]
Repository: CRAN
Date/Publication: 2025-09-14 11:30:24 UTC
Built: R 4.6.0; ; 2026-04-27 10:56:12 UTC; unix
