# Generated by roxygen2: do not edit by hand

export(add_interventions)
export(batch_migration)
export(calc_move_rate)
export(convert_prevalence_to_eir)
export(equilibrium_Imperial_decoupled)
export(equilibrium_SEI_Imperial)
export(equilibrium_SEI_SEIR)
export(equilibrium_SEI_SIS)
export(equilibrium_SEI_decoupled_human)
export(equilibrium_SEI_decoupled_mosy)
export(equilibrium_lifeycle)
export(get_shape)
export(human_Imperial_ODE)
export(imperial_model_param_list_create)
export(make_Q_Imperial)
export(make_Q_SEI)
export(movement_prob2rate)
export(sim_trajectory_CSV)
export(sim_trajectory_CSV_decoupled)
export(sim_trajectory_R)
export(sim_trajectory_R_decoupled)
export(solve_muAqua)
export(split_aggregate_CSV)
export(split_aggregate_CSV_decoupled)
export(spn_P_epiSEIR_network)
export(spn_P_epiSEIR_node)
export(spn_P_epiSIS_network)
export(spn_P_epiSIS_node)
export(spn_P_epi_decoupled_node)
export(spn_P_lifecycle_network)
export(spn_P_lifecycle_node)
export(spn_S)
export(spn_T_epiSEIR_network)
export(spn_T_epiSEIR_node)
export(spn_T_epiSIS_network)
export(spn_T_epiSIS_node)
export(spn_T_epi_decoupled_node)
export(spn_T_lifecycle_network)
export(spn_T_lifecycle_node)
export(spn_hazards)
export(spn_hazards_decoupled)
export(step_CLE)
export(step_DM)
export(step_ODE)
export(step_ODE_decoupled)
export(step_PTS)
export(step_PTS_decoupled)
export(summarize_eggs_geno)
export(summarize_eggs_stage)
export(summarize_females)
export(summarize_females_epi)
export(summarize_humans_epiImperial)
export(summarize_humans_epiSEIR)
export(summarize_humans_epiSIS)
export(summarize_larvae_geno)
export(summarize_larvae_stage)
export(summarize_males)
export(summarize_pupae_geno)
export(summarize_pupae_stage)
export(summarize_stats_CSV)
export(summarize_stats_CSV_decoupled)
export(track_hinf)
importFrom(Matrix,Matrix)
importFrom(Matrix,diag)
importFrom(Matrix,drop0)
importFrom(Matrix,solve)
importFrom(Matrix,sparseMatrix)
importFrom(Matrix,t)
importFrom(Matrix,which)
importFrom(deSolve,ode)
importFrom(statmod,gauss.quad.prob)
importFrom(stats,predict)
importFrom(stats,quantile)
importFrom(stats,rbinom)
importFrom(stats,rexp)
importFrom(stats,rnorm)
importFrom(stats,rpois)
importFrom(stats,runif)
importFrom(stats,setNames)
importFrom(stats,smooth.spline)
importFrom(stats,uniroot)
importFrom(utils,globalVariables)
importFrom(utils,setTxtProgressBar)
importFrom(utils,tail)
importFrom(utils,txtProgressBar)
importFrom(utils,write.table)
